# -*- coding: utf-8 -*-
"""
Program:
    Cute Thermowell
    (qt-thermowell)
    StepCalc.py

Author:
    Alex Borisov <>

Copyright (c) 2011 Alex Borisov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from xml.etree.ElementTree import ElementTree, Element, SubElement
#from xml.etree.ElementTree import ElementTree, Element, SubElement, tostring
#from xml.dom import minidom
import sys
import resources
import math
from decimal import Decimal

class StepCalc(object):
    def __init__(self, results_file,  parent = None):
        
        self.parent = parent
        self.results_stamp = ""
        self.results_file = results_file
        self.script_path = sys.path[0]

        self.version = ""

        #Dimensions calculation variables
        self.dim_test_result = "FAIL"
        self.dim_test_msg = "\nCalculation error occurred or parameters outside of Standard scope."

        #Frequency calculation variables
        self.Re = 0.0 # Renolds number
        self.Ns = 0.0 # Strouhal number
        self.fs = 0.0 # Frequency of vortex shedding
        self.fa = 0.0 # Approximate natural frequency of thermowell
        self.Da = 0.0 # Average outer diameter of thermowell
        self.I = 0.0 # Second moment of inertia
        self.m = 0.0 # Mass per unit length of thermowell
        self.Hf = 0.0 # Correction factor for deviations from a solid beam of uniform cross section
        self.Haf = 0.0 # Mass correction factor for fluid
        self.Has = 0.0 # Sensor mass correction factor
        self.fn = 0.0 # Lowest order natural frequency of thermowell with ideal support
        self.Km = 0.0 # Correction factor for rotational stiffness of thermowell support
        self.Hc = 0.0 # Mounting compliance frequency factor
        self.fcn = 0.0 # In situ natural frequency of thermowell
        self.Nsc = 0.0 # Scruton number
        self.damp_factor = 0.0005 # Intrinsic damping factor
        self.trans_freq_test_criterion = 0.0
        self.inline_freq_test_criterion = 0.0
        self.freq_test_result = "FAIL"
        self.freq_test_msg = "\nCalculation error occurred or parameters outside of Standard scope."
        
        #Cyclic drag stress calculation variables at in-line resonance velocity
        self.VIR = 0.0 # Fluid velocity for in-line resonance
        self.fs_VIR = 0.0 # Frequency of vortex shedding at in-line resonance velocity
        self.FMd_VIR = 1000 # Oscillating drag (in-line) magnification factor
        self.GSP_VIR = 0.0 # Max stress, no shielding from flow
        self.GRD_VIR = 0.0 # Max stress, no shielding from flow (step)
        self.Cd_VIR = 0.1
        self.Pd_VIR = 0.0 # Oscillating drag (in-line) force
        self.Sd_VIR = 0.0 # Oscillating drag stress
        self.Sd_step_VIR = 0.0 # Oscillating drag stress (step)
        self.Somax_VIR = 0.0 # Max oscillatory bending stress
        self.Somax_step_VIR = 0.0 # Max oscillatory bending stress (step)
        self.cd_stress_test_criterion = 0.0
        self.cd_stress_test_result = "FAIL"
        self.cd_stress_test_msg = "\nCalculation error occurred or parameters outside of Standard scope."
        self.cycl_html = False
        
        #Steady-state stress calculation variables
        self.FMd = 0.0 # Oscillating drag (in-line) magnification factor
        self.FMl = 0.0 # Oscillating lift (transverse) magnification factor
        self.GSP = 0.0 # Max stress, no shielding from flow
        self.GRD = 0.0 # Max stress, no shielding from flow (step)
        self.CD = 1.4
        self.PD = 0.0 #steady drag force
        self.Cd = 0.1
        self.Pd = 0.0 # Oscillating drag (in-line) force
        self.Cl = 1.0
        self.Pl = 0.0 # Oscillating lift (transverse) force
        self.SD = 0.0 # Steady-state drag stress
        self.SD_step = 0.0 # Steady-state drag stress (step)
        self.Sd = 0.0 # Oscillating drag stress
        self.Sd_step = 0.0 # Oscillating drag stress (step)
        self.Sl = 0.0 # Oscillating lift stress
        self.Sl_step = 0.0 # Oscillating lift stress (step)
        self.Sr = 0.0 #Radial pressure stress
        self.St = 0.0 # Tangential pressure stress
        self.St_step = 0.0 # Tangential pressure stress (step)
        self.Sa = 0.0 # Axial pressure stress
        self.Sa_step = 0.0 # Axial pressure stress (step)
        self.Smax = 0.0 # Max steady-state, combined hydrostatic fluid pressure and non-oscillating drag
        self.Smax_step = 0.0 # Max steady-state, combined hydrostatic fluid pressure and non-oscillating drag (step)
        self.VM_criterion = 0.0 # Von Mises criterion for failure
        self.VM_criterion_step = 0.0 # Von Mises criterion for failure (step)
        self.ss_stress_test_criterion = 0.0
        self.ss_stress_test_result = "FAIL"
        self.ss_stress_test_msg = "\nCalculation error occurred or parameters outside of Standard scope."
        
        #Dynamic stress calculation variables
        self.Kt = 0.0 # Stress concentration factor
        self.Kt_step = 0.0 # Stress concentration factor (step)
        self.Somax = 0.0 # Max oscillatory bending stress
        self.Somax_step = 0.0 # Max oscillatory bending stress (step)
        self.FT = 0.0 # Temperature correction factor
        self.FE = 1.0 # Environmental factor
        self.dyn_stress_test_criterion = 0.0
        self.dyn_stress_test_result = "FAIL"
        self.dyn_stress_test_msg = "\nCalculation error occurred or parameters outside of Standard scope."

        #Pressure calculation variables
        self.Pr = 0.0 # External pressure rating of thermowell
        self.Pf = 0.0 # Design pressure of flange supporting thermowell
        self.Pc = 0.0 # Design static pressure of thermowell shank
        self.Pt = 0.0 # Design Pressure of thermowell tip
        self.press_test_criterion = 0.0
        self.press_test_result = "FAIL"
        self.press_test_msg = "\nCalculation error occurred or parameters outside of Standard scope."

        #Thermowell data variables
        self.A = 0.0
        self.B = 0.0
        self.d = 0.0
        self.b = 0.0
        self.bs = 0.0
        self.t = 0.0
        self.L = 0.0
        self.Lo = 0.0
        self.Ls = 0.0
        
        self.TW_type = ""
        self.TW_connect = ""
        self.TW_install = ""
        self.T = ""
        self.TW_mat = ""
        self.flange_mat = ""
        self.flange_class = ""
        self.NDT = False
        
        self.TW_Eref = 0.0
        self.TW_Et = 0.0
        self.TW_dens = 0.0
        self.sensor_dens = 0.0
        self.TW_stress_limit = 0.0
        self.TW_fatigue_limit = 0.0
        self.TW_press_limit = 0.0
        
        #Pipe data variables
        self.pipe_mat = ""
        self.pipe_DN = ""
        self.pipe_sch = ""
        self.pipe_OD = 0.0
        self.pipe_WT = 0.0
        self.pipe_ID = 0.0
        self.TW_stress = True
        
        #Process data variables
        self.fluid_press = 0.0
        self.fluid_temp = 0.0
        self.fluid_rate = 0.0
        self.fluid_dens = 0.0
        self.fluid_spec_vol = 0.0
        self.fluid_velos = 0.0
        self.fluid_viscos = 0.0
                
    def Calculate(self, input_data):
        try:
            self.version = input_data.version

            self.A = input_data.A
            self.B = input_data.B
            self.d = input_data.d
            self.b = input_data.b
            self.bs = input_data.bs
            self.t = input_data.t
            self.L = input_data.L
            self.Lo = input_data.Lo
            self.Ls = input_data.Ls
            
            self.TW_type = input_data.TW_type
            self.TW_connect = input_data.TW_connect
            self.TW_install = input_data.TW_install
            self.T = input_data.T
            self.TW_mat = input_data.TW_mat
            self.flange_mat = input_data.flange_mat
            self.flange_class = input_data.flange_class
            self.NDT = input_data.NDT
            
            self.TW_Eref = input_data.TW_Eref
            self.TW_Et = input_data.TW_Et
            self.TW_dens = input_data.TW_dens
            self.sensor_dens = input_data.sensor_dens
            self.TW_stress_limit = input_data.TW_stress_limit
            self.TW_fatigue_limit = input_data.TW_fatigue_limit
            self.TW_press_limit = input_data.TW_press_limit
            
            self.pipe_mat = input_data.pipe_mat
            self.pipe_DN = input_data.pipe_DN
            self.pipe_sch = input_data.pipe_sch
            self.pipe_OD = input_data.pipe_OD
            self.pipe_WT = input_data.pipe_WT
            self.pipe_ID = input_data.pipe_ID
            self.TW_stress = input_data.TW_stress
    
            self.fluid_press = input_data.fluid_press
            self.fluid_temp = input_data.fluid_temp
            self.fluid_rate = input_data.fluid_rate
            self.fluid_dens = input_data.fluid_dens
            self.fluid_spec_vol = input_data.fluid_spec_vol
            self.fluid_velos = input_data.fluid_velos
            self.fluid_viscos = input_data.fluid_viscos
            
            if self.TW_install == ("45" + u"\u00B0 " + "Angle") or self.TW_install == ("90" + u"\u00B0 " + "Elbow"):
                self.L = self.L * math.sin(math.pi/ 4)
                self.Lo = self.Lo * math.sin(math.pi/ 4)
                self.Ls = self.Ls * math.sin(math.pi/ 4)
            
            self.FT = self.TW_Et / self.TW_Eref
    
            if self.b == 0 or (self.A / self.b) >= 33:
                self.Kt = 2.2
            elif self.TW_connect == "Threaded":
                self.Kt = 2.3
            else:
                self.Kt = 1.1 + 0.033 * (self.A / self.b)
            
            if self.bs == 0 or (self.B / self.bs) >= 33:
                self.Kt_step = 2.2
            elif self.TW_connect == "Threaded":
                self.Kt_step = 2.3
            else:
                self.Kt_step = 1.1 + 0.033 * (self.B / self.bs)
            
            if not self.DimTest():
                self.results_stamp = self.SaveResults()
                return "Fail"
            elif not self.PressCalc():
                self.freq_test_result = "N/A"
                self.ss_stress_test_result = "N/A"
                self.dyn_stress_test_result = "N/A"
                self.freq_test_msg = "Calculation was not performed."
                self.ss_stress_test_msg = "Calculation was not performed."
                self.dyn_stress_test_msg = "Calculation was not performed."
                self.results_stamp = self.SaveResults()
                return "Fail"
            elif self.fluid_velos < 0.64 and (self.A - self.d) >= 0.00955 and self.L <= 0.61 and self.B >= 0.0127 and self.A >= self.B and self.TW_stress_limit >= 69000000 and self.TW_fatigue_limit >= 21000000 and self.TW_stress == False:
                self.dim_test_result = "PASS"
                self.dim_test_msg = "Thermowell dimensions within Standard scope."
                self.freq_test_result = "PASS"
                self.freq_test_msg = "Natural frequency calculation does not need to be performed."
                self.freq_test_msg += "\nFrequency of thermowell vortex shedding is less than natural frequency limits of in-situ mounted thermowell."
                self.ss_stress_test_result = "PASS"
                self.ss_stress_test_msg = "Steady-state stress calculation does not need to be performed."
                self.ss_stress_test_msg += "\nSteady-state stress of thermowell is less than max allowable stress limits of thermowell material."
                self.dyn_stress_test_result = "PASS"
                self.dyn_stress_test_msg = "Dynamic stress calculation does not need to be performed."
                self.dyn_stress_test_msg += "\nDynamic stress of thermowell is less than max fatigue endurance limits of thermowell material."
                self.press_test_result = "PASS"
                self.press_test_msg = "External pressure rating of thermowell is greater than max operating pressure."
                self.results_stamp = self.SaveResults()
                return "Pass"
            elif self.FreqCalc() and self.SSStressCalc() and self.DynStressCalc():
                self.results_stamp = self.SaveResults()
                return "Pass"
            else:
                self.results_stamp = self.SaveResults()
                return "Fail"
        except Exception, inst:
            return "Error"
            
    def DimTest(self):
        if self.B == 0.01270 and (self.L >= 0.127 and self.L <= 0.6096) and (self.d >= 0.0061 and self.d <= 0.0067) and ((self.B / self.A) >= 0.5 and (self.B / self.A) <= 0.8) and ((self.Ls / self.L) > 0 and (self.Ls / self.L) < 0.6) and (((self.B - self.d) / 2) >= 0.003):
            insert_test_result = self.CheckTWInsertion()
            if insert_test_result == "PASS":
                self.dim_test_result = "PASS"
                self.dim_test_msg = "Thermowell dimensions within Standard scope."
                return True
            elif insert_test_result == "WARN":
                self.dim_test_result = "WARN"
                self.dim_test_msg = "Thermowell dimensions within Standard scope. However thermowell tip is not located at center of pipe"
                return True
            elif insert_test_result == "FAIL":
                self.dim_test_result = "FAIL"
                self.dim_test_msg = "Thermowell may be too short or too long."
                self.freq_test_result = "N/A"
                self.freq_test_msg = "Calculation was not performed."
                self.ss_stress_test_result = "N/A"
                self.ss_stress_test_msg = "Calculation was not performed."
                self.dyn_stress_test_result = "N/A"
                self.dyn_stress_test_msg = "Calculation was not performed."
                self.press_test_result = "N/A"
                self.press_test_msg = "Calculation was not performed."
                return False
        elif self.B == 0.02223 and (self.L >= 0.127 and self.L <= 0.6096) and (self.d >= 0.0061 and self.d <= 0.0067) and ((self.B / self.A) >= 0.583 and (self.B / self.A) <= 0.875) and ((self.Ls / self.L) > 0 and (self.Ls / self.L) < 0.6) and (((self.B - self.d) / 2) >= 0.003):
            insert_test_result = self.CheckTWInsertion()
            if insert_test_result == "PASS":
                self.dim_test_result = "PASS"
                self.dim_test_msg = "Thermowell dimensions within Standard scope."
                return True
            elif insert_test_result == "WARN":
                self.dim_test_result = "WARN"
                self.dim_test_msg = "Thermowell dimensions within Standard scope. However thermowell tip is not located at center of pipe"
                return True
            elif insert_test_result == "FAIL":
                self.dim_test_result = "FAIL"
                self.dim_test_msg = "Thermowell may be too short or too long."
                self.freq_test_result = "N/A"
                self.freq_test_msg = "Calculation was not performed."
                self.ss_stress_test_result = "N/A"
                self.ss_stress_test_msg = "Calculation was not performed."
                self.dyn_stress_test_result = "N/A"
                self.dyn_stress_test_msg = "Calculation was not performed."
                self.press_test_result = "N/A"
                self.press_test_msg = "Calculation was not performed."
                return False
        else:
            self.dim_test_result = "FAIL"
            self.dim_test_msg = "Thermowell dimensions outside of Standard scope."
            self.freq_test_result = "N/A"
            self.freq_test_msg = "Calculation was not performed."
            self.ss_stress_test_result = "N/A"
            self.ss_stress_test_msg = "Calculation was not performed."
            self.dyn_stress_test_result = "N/A"
            self.dyn_stress_test_msg = "Calculation was not performed."
            self.press_test_result = "N/A"
            self.press_test_msg = "Calculation was not performed."
            return False

    def CheckTWInsertion(self):
        if (self.L - self.Lo) > (0.8 * self.pipe_ID / 2) and (self.L - self.Lo) < (1.2 * self.pipe_ID / 2): # TW tip within 20 % of pipe center
            return "PASS"
        elif (self.L - self.Lo) > (0.5 * self.pipe_ID / 2) and (self.L - self.Lo) < (1.5 * self.pipe_ID / 2): # TW tip within 20-50 % of pipe center
            return "WARN"
        else:
            return "FAIL"
        
    def FreqCalc(self):
        pipe_rad = self.pipe_ID / 2 # Pipe radius
        pipe_area = math.pi * math.pow(pipe_rad, 2) # Pipe area
        if self.fluid_velos != 0 and self.fluid_dens != 0:
            self.fluid_rate = self.fluid_dens * self.fluid_velos * pipe_area
        dens_velos = self.fluid_rate / pipe_area
        
        if self.fluid_velos == 0 and self.fluid_dens != 0 and self.fluid_rate > 0:
            self.fluid_velos = self.fluid_rate / (pipe_area * self.fluid_dens)
            
        if (self.fluid_velos == 0 or self.fluid_dens == 0) and self.fluid_rate > 0:
            self.Re = self.B * dens_velos / self.fluid_viscos
        else:
            self.Re = (self.fluid_velos * self.B * self.fluid_dens) / self.fluid_viscos
        
        if self.Re >= 22 and self.Re < 1300:
            self.Ns = 0.22 * (1 - 22 / self.Re)
        elif self.Re >= 1300 and self.Re < 500000:
            self.Ns = 0.213 - 0.0248 * math.pow(math.log10(self.Re / 1300), 2) + 0.0095 * math.pow(math.log10(self.Re / 1300), 3)
        else: 
            self.Ns = 0.22
        
        self.fs = self.fsCalc(self.fluid_velos)
        
        self.Da = self.A
        self.I = (math.pi * (math.pow(self.Da, 4) - math.pow(self.d, 4))) / 64
        self.m = (self.TW_dens * math.pi * (math.pow(self.Da, 2) - math.pow(self.d, 2))) / 4
        self.fa = (math.pow(1.875, 2) / (2 * math.pi)) * math.sqrt((self.TW_Et * self.I) / self.m) * (1 / math.pow(self.L, 2))
        
        if self.B == 0.01270:
            C1 = 1.407
            C2 = -0.839
            C3 = -0.022
            C4 = 1.022
            C5 = -2.228
            C6 = 1.594
            C7 = 1.313
            C8 = 0.362
            C9 = 8.299
            C10 = -5.376
        elif self.B == 0.02223:
            C1 = 1.410
            C2 = -0.949
            C3 = -0.091
            C4 = 1.132
            C5 = -1.714
            C6 = 0.865
            C7 = 0.861
            C8 = 1.000
            C9 = 9.275
            C10 = -7.466
        
        Y1 = (C1 * (self.A / self.B) + C2) * (self.Ls / self.L) + (C3 * (self.A / self.B) + C4)
        Y2 = (C5 * (self.A / self.B) + C6) * (self.Ls / self.L) + (C7 * (self.A / self.B) + C8)
        b = (C9 * (self.A / self.B) + C10)
        self.Hf = math.pow((math.pow(Y1, -b) + math.pow(Y2, -b)), -(1 / b))
        
        self.Haf = 1 - self.fluid_dens / (2 * self.TW_dens)
        
        self.Has = 1 - (self.sensor_dens / (2 * self.TW_dens)) * (1 / (math.pow((self.Da / self.d), 2) - 1))
        
        self.fn = self.Hf * self.Haf * self.Has * self.fa
        
        self.Km = (self.TW_Et / 0.787) * math.pow((self.A / 2), 3)
        
        if self.TW_connect == "Threaded":
            self.Hc = 1 - 0.9 * (self.A / self.L)
        else:
            self.Hc = 1 - (1 / (self.Km / self.TW_Et)) * ((math.pi * (math.pow(self.A, 4) - math.pow(self.d, 4))) / (32 * self.L * math.pow((1 + 1.5 * (self.b / self.A)), 2)))
        
        self.fcn = self.Hc * self.fn
        
        self.Nsc = math.pow(math.pi, 2) * self.damp_factor * (self.TW_dens / self.fluid_dens) * (1 - math.pow((self.d / self.B), 2))
        
        if self.Nsc > 2.5 and self.Nsc <= 64 and self.Re < 100000:
            self.trans_freq_test_criterion = 0.8 * self.fcn
            if self.fs < self.trans_freq_test_criterion:
                self.freq_test_result = "PASS"
                self.freq_test_msg = "Frequency of thermowell vortex shedding is less than natural frequency limits of in-situ mounted thermowell."
                return True
            else:
                self.freq_test_result = "FAIL"
                self.freq_test_msg = "Frequency of thermowell vortex shedding is greater than natural frequency limits of in-situ mounted thermowell."
                self.ss_stress_test_result = "N/A"
                self.ss_stress_test_msg = "Calculation was not performed."
                self.dyn_stress_test_result = "N/A"
                self.dyn_stress_test_msg = "Calculation was not performed."
                return False
        elif self.Nsc <= 2.5 or self.Re >= 100000:            
            self.inline_freq_test_criterion = 0.4 * self.fcn
            if self.fs < self.inline_freq_test_criterion:
                self.freq_test_result = "PASS"
                self.freq_test_msg = "Frequency of thermowell vortex shedding is less than natural frequency limits of in-situ mounted thermowell."
                return True
            elif self.CDStressCalc():
                self.inline_freq_test_criterion = 0.6 * self.fcn
                self.trans_freq_test_criterion = 0.8 * self.fcn
                if self.fs > self.inline_freq_test_criterion and self.fs < self.trans_freq_test_criterion:
                    self.freq_test_result = "PASS"
                    self.freq_test_msg = "Frequency of thermowell vortex shedding is less than natural frequency limits of in-situ mounted thermowell."
                    return True
                else:
                    self.freq_test_result = "FAIL"
                    self.freq_test_msg = "Frequency of thermowell vortex shedding is greater than natural frequency limits of in-situ mounted thermowell."
                    self.ss_stress_test_result = "N/A"
                    self.ss_stress_test_msg = "Calculation was not performed."
                    self.dyn_stress_test_result = "N/A"
                    self.dyn_stress_test_msg = "Calculation was not performed."
                    return False
            else:
                self.freq_test_result = "FAIL"
                self.freq_test_msg = "Frequency of thermowell vortex shedding is greater than natural frequency limits of in-situ mounted thermowell."
                self.ss_stress_test_result = "N/A"
                self.ss_stress_test_msg = "Calculation was not performed."
                self.dyn_stress_test_result = "N/A"
                self.dyn_stress_test_msg = "Calculation was not performed."
                return False
        else:
            return False
    
    def fsCalc(self, V):        
        fs = self.Ns * (V / self.B)
        return fs

    def CDStressCalc(self):        
        self.cycl_html = True
        
        Reo = 1300
        R = math.log10(self.Re / Reo)
        aR = 0.0285 * math.pow(R, 2) - 0.0496 * R
        if self.Re >= 22 and self.Re < 1300:
            self.VIR = ((self.B * self.fcn) / (2 * self.Ns)) * (1 - (22 * self.fluid_viscos) / (self.B * self.fluid_dens * self.fluid_velos)) + ((22 * self.fluid_viscos) / (self.B * self.fluid_dens))
        elif self.Re >= 1300 and self.Re < 500000:
            self.VIR = ((self.B * self.fcn) / (2 * self.Ns)) * (1 - (aR / self.Ns) * math.log10((self.B * self.fcn) / (2 * self.Ns * self.fluid_velos)))
        elif self.Re >= 500000 and self.Re < 10000000:
            self.VIR = (self.B * self.fcn) / (2 * self.Ns)

        self.Pd_VIR = self.OscillatingDrag(self.fluid_dens, self.VIR)
        
        if self.Lo == 0:
            self.GSP_VIR = ((16 * math.pow(self.L, 2)) / (math.pi * math.pow(self.A, 2) * (1 - math.pow((self.d / self.A), 4)))) * ((self.B / self.A) + (1 - self.B / self.A) * math.pow(1 - self.Ls / self.L, 2))
            self.GRD_VIR = ((16 * math.pow(self.Ls, 2)) / (math.pi * math.pow(self.B, 2) * (1 - math.pow((self.d / self.B), 4))))
        elif self.Lo < (self.L - self.Ls):
            self.GSP_VIR = ((16 * math.pow(self.L, 2)) / (math.pi * math.pow(self.A, 2) * (1 - math.pow((self.d / self.A), 4)))) * ((self.B / self.A) + (1 - self.B / self.A) * math.pow(1 - self.Ls / self.L, 2) - math.pow((self.Lo / self.L), 2))
            self.GRD_VIR = ((16 * math.pow(self.Ls, 2)) / (math.pi * math.pow(self.B, 2) * (1 - math.pow((self.d / self.B), 4))))
        else:
            self.GSP_VIR = ((16 * self.B * math.pow(self.L, 2)) / (math.pi * math.pow(self.A, 3) * (1 - math.pow((self.d / self.A), 4)))) * (1 - math.pow((self.Lo / self.L), 2))
            self.GRD_VIR = ((16 * math.pow(self.Ls, 2)) / (math.pi * math.pow(self.B, 2))) * (1 - self.Lo / self.L) * (2 * (self.Ls / self.L) - 1 + (self.Lo / self.L))
        
        self.Sd_VIR = self.GSP_VIR * self.FMd_VIR * self.Pd_VIR
        self.Sd_step_VIR = self.GRD_VIR * self.FMd_VIR * self.Pd_VIR
        
        self.Somax_VIR = self.Kt * self.Sd_VIR
        self.Somax_step_VIR = self.Kt_step * self.Sd_step_VIR
        
        self.cd_stress_test_criterion = self.FT * self.FE * self.TW_fatigue_limit
        
        if self.Somax_VIR < self.cd_stress_test_criterion and self.Somax_step_VIR < self.cd_stress_test_criterion:
            self.cd_stress_test_result = "PASS"
            self.cd_stress_test_msg = "Cyclic stress of thermowell at in-line resosnance velocity is less than max allowable stress limits of thermowell material."
            return True
        else:
            self.cd_stress_test_result = "FAIL"
            self.cd_stress_test_msg = "Cyclic stress of thermowell at in-line resosnance velocity is greater than max allowable stress limits of thermowell material."
            return False
        
    def SSStressCalc(self): 
        rd = (2 * self.fs) / self.fcn
        self.FMd = 1 / (1 - math.pow(rd, 2))
        
        rl = self.fs / self.fcn
        self.FMl = 1 / (1 - math.pow(rl, 2))
        
        if self.Lo == 0:
            self.GSP = ((16 * math.pow(self.L, 2)) / (math.pi * math.pow(self.A, 2) * (1 - math.pow((self.d / self.A), 4)))) * ((self.B / self.A) + (1 - self.B / self.A) * math.pow(1 - self.Ls / self.L, 2))
            self.GRD = ((16 * math.pow(self.Ls, 2)) / (math.pi * math.pow(self.B, 2) * (1 - math.pow((self.d / self.B), 4))))
        elif self.Lo < (self.L - self.Ls):
            self.GSP = ((16 * math.pow(self.L, 2)) / (math.pi * math.pow(self.A, 2) * (1 - math.pow((self.d / self.A), 4)))) * ((self.B / self.A) + (1 - self.B / self.A) * math.pow(1 - self.Ls / self.L, 2) - math.pow((self.Lo / self.L), 2))
            self.GRD = ((16 * math.pow(self.Ls, 2)) / (math.pi * math.pow(self.B, 2) * (1 - math.pow((self.d / self.B), 4))))
        else:
            self.GSP = ((16 * self.B * math.pow(self.L, 2)) / (math.pi * math.pow(self.A, 3) * (1 - math.pow((self.d / self.A), 4)))) * (1 - math.pow((self.Lo / self.L), 2))
            self.GRD = ((16 * math.pow(self.Ls, 2)) / (math.pi * math.pow(self.B, 2))) * (1 - self.Lo / self.L) * (2 * (self.Ls / self.L) - 1 + (self.Lo / self.L))
        
        self.PD = self.SteadyDrag(self.fluid_dens, self.fluid_velos)

        self.Pd = self.OscillatingDrag(self.fluid_dens, self.fluid_velos)

        self.Pl = self.OscillatingLift(self.fluid_dens, self.fluid_velos)

        self.SD = self.GSP * self.PD
        self.SD_step = self.GRD * self.PD
        
        self.Sd = self.GSP * self.FMd * self.Pd
        self.Sd_step = self.GRD * self.FMd * self.Pd
        
        self.Sl = self.GSP * self.FMl * self.Pl
        self.Sl_step = self.GRD * self.FMl * self.Pl
        
        self.Sr = self.fluid_press
        
        self.St = self.fluid_press * (1 + math.pow((self.d / self.A), 2)) / (1 - math.pow((self.d / self.A), 2))
        self.St_step = self.fluid_press * (1 + math.pow((self.d / self.B), 2)) / (1 - math.pow((self.d / self.B), 2))
        
        self.Sa = self.fluid_press / (1 - math.pow((self.d / self.A), 2))
        self.Sa_step = self.fluid_press / (1 - math.pow((self.d / self.B), 2))
        
        self.Smax = self.SD + self.Sa
        self.Smax_step = self.SD_step + self.Sa_step
        
        self.VM_criterion = math.sqrt((math.pow((self.Smax - self.Sr), 2) + math.pow((self.Smax - self.St), 2) + math.pow((self.St - self.Sr), 2)) / 2)
        self.VM_criterion_step = math.sqrt((math.pow((self.Smax_step - self.Sr), 2) + math.pow((self.Smax_step - self.St_step), 2) + math.pow((self.St_step - self.Sr), 2)) / 2)
        
        self.ss_stress_test_criterion = 1.5 * self.TW_stress_limit
        if self.VM_criterion < self.ss_stress_test_criterion and self.VM_criterion_step < self.ss_stress_test_criterion:
            self.ss_stress_test_result = "PASS"
            self.ss_stress_test_msg = "Steady-state stress of thermowell is less than max allowable stress limits of thermowell material."
            return True
        else:
            self.ss_stress_test_result = "FAIL"
            self.ss_stress_test_msg = "Steady-state stress of thermowell is greater than max allowable stress limits of thermowell material."
            return False
        
    def DynStressCalc(self): 
        self.Somax = self.Kt * math.sqrt(math.pow(self.Sd, 2) + math.pow(self.Sl, 2))
        self.Somax_step = self.Kt_step * math.sqrt(math.pow(self.Sd_step, 2) + math.pow(self.Sl_step, 2))
        
        self.dyn_stress_test_criterion = self.FT * self.FE * self.TW_fatigue_limit
        if self.Somax < self.dyn_stress_test_criterion and self.Somax_step < self.dyn_stress_test_criterion:
            self.dyn_stress_test_result = "PASS"
            self.dyn_stress_test_msg = "Dynamic stress of thermowell is less than max fatigue endurance limits of thermowell material."
            return True
        else:
            self.dyn_stress_test_result = "FAIL"
            self.dyn_stress_test_msg = "Dynamic stress of thermowell is greater than max fatigue endurance limits of thermowell material."
            return False        
        
    def PressCalc(self):
        if self.t < 0.003:
            self.press_test_result = "FAIL"
            self.press_test_msg = "Thermowell tip thickness is less than min wall thickness of thermowell shank."
            return False
        self.Pf = self.TW_press_limit
        self.Pc = 0.66 * self.TW_stress_limit * ((2.167 / ((2 * self.B) / (self.B - self.d))) - 0.0833)
        self.Pt = (self.TW_stress_limit / 0.13) * math.pow((self.t / self.d), 2)
        if self.TW_connect == "Flanged":
            self.Pr = min(self.Pc, self.Pt, self.Pf)
        else:
            self.Pr = min(self.Pc, self.Pt)
        self.press_test_criterion = self.fluid_press
        if self.Pf < 103000000:
            if self.Pr > self.press_test_criterion:
                self.press_test_result = "PASS"
                self.press_test_msg = "External pressure rating of thermowell is greater than max operating pressure."
                return True
            else:
                self.press_test_result = "FAIL"
                self.press_test_msg = "External pressure rating of thermowell is less than max operating pressure."
                return False
        else:
            self.press_test_result = "N/A"
            self.press_test_msg = "Not implemented. Further investigation required at such high pressure."
            return False
        
    def SteadyDrag(self, fluid_dens, V):
        CD = 1.4
        PD = 0.0 #steady drag force
        PD = 0.5 * fluid_dens * CD * math.pow(V, 2)
        return PD

    def OscillatingDrag(self, fluid_dens, V):
        Cd = 0.1
        Pd = 0.0 # Oscillating drag (in-line) force
        Pd = 0.5 * fluid_dens * Cd * math.pow(V, 2)
        return Pd

    def OscillatingLift(self, fluid_dens, V):
        Cl = 1.0
        Pl = 0.0 # Oscillating lift (transverse) force
        Pl = 0.5 * fluid_dens * Cl * math.pow(V, 2)
        return Pl
    
    def SetDecimals(self, str_value):
        rnd_value = round(Decimal(str(str_value)), 5)
        return str(rnd_value)

    def SaveResults(self):
        stamp = QDateTime().currentDateTime().toString("ddMMyyyyhhmmss")
        html = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">\n'
        html += '<html xmlns="http://www.w3.org/1999/xhtml">\n'
        html += '<head>\n'
        html += '<meta http-equiv="content-type" content="text/html; charset=UTF-8" />\n'
        html += '<meta name="stamp" content="' + stamp + '" /> \n'
        html += '<title>Thermowell Calculation Results</title>\n'
        html += '<style type="text/css">\n'
        html += 'body {\n'
        html += 'font-size: 8pt;\n'
        #html += 'color: blue;\n'
        html += '}\n'
        html += 'table {\n'
        html += 'border-collapse: collapse;\n'
        html += '}\n'
        html += '</style>\n'
        html += '</head>\n'
        
        html += '<body>\n'
        html += '<h2>Thermowell Design Calculations</h2>\n'
        html += '<h3>ASME PTC 19.3 TW - 2010</h3>'
        html += 'Date: ' + QDateTime().currentDateTime().toString("dd MMM yyyy") + '\n'
        html += '<br /><br />\n'
        
        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>'
        html += '<td width="15%"><b>Calculation Title</b></td>\n'
        html += '<td></td>\n'
        html += '<td width="15%"><b>Calculation No.</b></td>\n'
        html += '<td></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td><b>Project Title</b></td>\n'
        html += '<td></td>\n'
        html += '<td><b>Project No.</b></td>\n'
        html += '<td></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td><b>Client</b></td>\n'
        html += '<td></td>'
        html += '<td><b>Phase/CTR</b></td>\n'
        html += '<td></td>\n'
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br /><br />\n'
        
        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td><b>Calculation Objectives</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td>To verify that the thermowell vortex shedding (wake frequency), steady-state stress, dynamic stress and hydrostatic pressure limits are not exceeded with the given process conditions.</td>\n'
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br />\n'

        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td><b>Assumptions</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td></td>\n'
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br />\n'
        
        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td><b>References</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td>ASME PTC 19.3 TW-2010<br />ASME II, ASME VIII<br />ASME B16.5, ASME B31.1, ASME B31.3, ASME B36.10<br />Datasheet(s)<br />Drawing(s)</td>\n'
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br />\n'

        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td><b>Conclusions</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td></td>\n'
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br /><br />\n'
        
        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td align="center" colspan="4"><b>Thermowell Selection</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Thermowell Type'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.TW_type
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'Thermowell Connection'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.TW_connect
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Thermowell Installation'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.TW_install
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'T<sub>Design</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.T) + ' &#x0B0;C'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Thermowell Material'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.TW_mat
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'Flange Material'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.flange_mat
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Flange Class'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.flange_class
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'NDT Required'
        html += '</td>\n' 
        html += '<td align="center">'
        html += str(self.NDT)
        html += '</td>\n' 
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br /><br />\n'

        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td align="center" colspan="4"><b>Thermowell Dimensions</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'A'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.A * 1000) + ' mm'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'B'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.B * 1000) + ' mm'
        html += '</td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'd'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.d * 1000) + ' mm'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'b'
        html += '</td>\n'
        html += '<td align="center">'
        html += self.SetDecimals(self.b * 1000) + ' mm'
        html += '</td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 't'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.t * 1000) + ' mm'
        html += '</td>\n'
        html += '<td align="center">'
        html += 'L'
        html += '</td>\n' 
        html += '<td align="center">'     
        html += self.SetDecimals(self.L * 1000) + ' mm'
        html += '</td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'L<sub>0</sub>'
        html += '</td>\n'
        html += '<td align="center">'
        html += self.SetDecimals(self.Lo * 1000) + ' mm'
        html += '</td>\n'
        html += '<td align="center">'
        html += 'b<sub>S</sub>'
        html += '</td>\n'
        html += '<td align="center">'
        html += self.SetDecimals(self.bs * 1000) + ' mm'
        html += '</td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'L<sub>S</sub>'
        html += '</td>\n'
        html += '<td align="center">'
        html += self.SetDecimals(self.Ls * 1000) + ' mm'
        html += '</td>\n'
        html += '<td colspan="2" width="50%">'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Test Result'
        html += '</td>\n' 
        if  self.dim_test_result == "PASS":
            color = "darkgreen"
            fontcolor = "white"
        elif self.dim_test_result == "WARN":
            color = "yellow"
            fontcolor = "red"
        elif self.dim_test_result == "N/A":
            color = "darkgrey"
            fontcolor = "white"
        else:
            color = "red"
            fontcolor = "white"
        html += '<td align="center" valign="middle" style="background-color:  ' + color + '; color:  ' + fontcolor + ';">'
        html += '<b>' + self.dim_test_result + '</b>'
        html += '</td>\n' 
        html += '<td align="center" colspan="2" align="left" width="50%">'
        html += self.dim_test_msg
        html += '</td>\n' 
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br /><br />\n'
        
        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td align="center" colspan="4"><b>Thermowell Data</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'E<sub>ref</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.TW_Eref / 1000000000) + ' GPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'E<sub>T</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.TW_Et / 1000000000) + ' GPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += '&#x3C1;<sub>m</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.TW_dens) + ' kg/m<sup>3</sup>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += '&#x3C1;<sub>s</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.sensor_dens) + ' kg/m<sup>3</sup>'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'S'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.TW_stress_limit / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'S<sub>f</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.TW_fatigue_limit / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'P<sub>f</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Pf / 1000) + ' kPa'
        html += '</td>\n' 
        html += '<td colspan="2" width="50%">'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br /><br />\n'
        
        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td align="center" colspan="4"><b>Pipe Data</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Material'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.pipe_mat
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'Nominal Diameter'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.pipe_DN + ' mm'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Schedule'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.pipe_sch
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'Outside Diameter'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.pipe_OD * 1000) + ' mm'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Wall Thickness'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.pipe_WT * 1000) + ' mm'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'Inside Diameter'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.pipe_ID * 1000) + ' mm'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br /><br />\n'

        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td align="center" colspan="4"><b>Process Data</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'P'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.fluid_press / 1000) + ' kPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'T'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.fluid_temp) + ' &#x0B0;C'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'F'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.fluid_rate) + ' kg/s'
        html += '</td>\n' 
        html += '<td align="center">'
        html += '&#x3C1;'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.fluid_dens) + ' kg/m<sup>3</sup>'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'v'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.fluid_spec_vol) + ' m<sup>3</sup>/kg'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'V'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.fluid_velos) + ' m/s'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += '&#x3BC;'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.fluid_viscos * 1000) + ' cP'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'Corrosion / Embrittlement'
        html += '</td>\n' 
        html += '<td align="center">'
        html += str(self.TW_stress)
        html += '</td>\n' 
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br /><br />\n'

        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td align="center" colspan="4"><b>Vortex Shedding (Wake Frequency) Calculation</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'R<sub>e</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Re)
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'N<sub>s</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Ns)
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'f<sub>a</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.fa) + ' Hz'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'D<sub>a</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Da * 1000) + ' mm'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'I'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.I) + ' kg&#x0B7;m<sup>2</sup>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'm'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.m) + ' kg/m'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'H<sub>f</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Hf)
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'H<sub>a,f</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Haf)
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'H<sub>a,s</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Has)
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'f<sub>n</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.fn) + ' Hz'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'K<sub>M</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Km) + ' N&#x0B7;m/rad'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'H<sub>c</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Hc)
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'f<sub>c,n</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.fcn) + ' Hz'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'N<sub>Sc</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Nsc)
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += '&#x3B6;'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.damp_factor)
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'f<sub>S</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.fs) + ' Hz'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'f<sub>S (transverse)</sub> Test Criterion'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.inline_freq_test_criterion) + ' Hz'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'f<sub>S (in-line)</sub> Test Criterion'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.trans_freq_test_criterion) + ' Hz'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Test Result'
        html += '</td>\n' 
        if  self.freq_test_result == "PASS":
            color = "darkgreen"
        elif self.freq_test_result == "N/A":
            color = "darkgrey"
        else:
            color = "red"
        html += '<td align="center" valign="middle" style="background-color:  ' + color + '; color: white;">'
        html += '<b>' + self.freq_test_result + '</b>'
        html += '</td>\n' 
        html += '<td align="center" colspan="2" align="left" width="50%">'
        html += self.freq_test_msg
        html += '</td>\n' 
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br /><br />\n'
        
        if self.cycl_html:
            html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
            html += '<tr>\n'
            html += '<td align="center" colspan="4"><b>Cyclic Stress Calculation at In-Line Resonance Velocity</b></td>\n'
            html += '</tr>\n'
            html += '<tr>\n'
            html += '<td align="center" width="20%"><b>Property</b></td>\n'
            html += '<td align="center" width="30%"><b>Value</b></td>\n'
            html += '<td align="center" width="20%"><b>Property</b></td>\n'
            html += '<td align="center" width="30%"><b>Value</b></td>\n'
            html += '</tr>\n'
            html += '<tr>\n'
            html += '<td align="center">'
            html += 'V<sub>IR</sub>'
            html += '</td>\n' 
            html += '<td align="center">'
            html += self.SetDecimals(self.VIR) + ' m/s'
            html += '</td>\n' 
            html += '<td align="center">'
            html += 'C<sub>d</sub>'
            html += '</td>\n' 
            html += '<td align="center">'
            html += self.SetDecimals(self.Cd)
            html += '</td>\n' 
            html += '</tr>\n'
            html += '<tr>\n'
            html += '<td align="center">'
            html += 'P<sub>d (VIR)</sub>'
            html += '</td>\n' 
            html += '<td align="center">'
            html += self.SetDecimals(self.Pd_VIR) + ' Pa'
            html += '</td>\n'                 
            html += '<td align="center">'
            html += 'F<sub>Md (VIR)</sub>'
            html += '</td>\n' 
            html += '<td align="center">'
            html += self.SetDecimals(self.FMd_VIR)
            html += '</td>\n' 
            html += '</tr>\n'
            html += '<tr>\n'
            html += '<td align="center">'
            html += 'G<sub>SP (VIR)</sub>'
            html += '</td>\n' 
            html += '<td align="center">'
            html += self.SetDecimals(self.GSP_VIR)
            html += '</td>\n' 
            html += '<td align="center">'
            html += 'G<sub>RD (VIR)</sub>'
            html += '</td>\n' 
            html += '<td align="center">'
            html += self.SetDecimals(self.GRD_VIR)
            html += '</td>\n' 
            html += '</tr>\n'
            html += '<tr>\n'
            html += '<td align="center">'
            html += 'S<sub>d (VIR)</sub>'
            html += '</td>\n' 
            html += '<td align="center">'
            html += self.SetDecimals(self.Sd_VIR / 1000000) + ' MPa'
            html += '</td>\n' 
            html += '<td align="center">'
            html += 'S<sub>d step(VIR)</sub>'
            html += '</td>\n' 
            html += '<td align="center">'
            html += self.SetDecimals(self.Sd_step_VIR / 1000000) + ' MPa'
            html += '</td>\n' 
            html += '</tr>\n'
            html += '<tr>\n'
            html += '<td align="center">'
            html += 'K<sub>t</sub>'
            html += '</td>\n' 
            html += '<td align="center">'
            html += self.SetDecimals(self.Kt)
            html += '</td>\n' 
            html += '<td align="center">'
            html += 'K<sub>t step</sub>'
            html += '</td>\n' 
            html += '<td align="center">'
            html += self.SetDecimals(self.Kt_step)
            html += '</td>\n' 
            html += '</tr>\n'
            html += '<tr>\n'
            html += '<td align="center">'
            html += 'F<sub>T</sub>'
            html += '</td>\n' 
            html += '<td align="center">'
            html += self.SetDecimals(self.FT)
            html += '</td>\n'
            html += '<td align="center">'
            html += 'S<sub>o,max (VIR)</sub>'
            html += '</td>\n'
            html += '<td align="center">'
            html += self.SetDecimals(self.Somax_VIR / 1000000) + ' MPa'
            html += '</td>\n'
            html += '</tr>\n'
            html += '<tr>\n'
            html += '<td align="center">'
            html += 'S<sub>o,max step(VIR)</sub>'
            html += '</td>\n'
            html += '<td align="center">'
            html += self.SetDecimals(self.Somax_step_VIR / 1000000) + ' MPa'
            html += '</td>\n'
            html += '<td align="center">'
            html += 'S<sub>o,max (VIR)</sub> Test Criterion'
            html += '</td>\n'
            html += '<td align="center">'
            html += self.SetDecimals(self.cd_stress_test_criterion / 1000000) + ' MPa'
            html += '</td>\n'
            html += '</tr>\n'
            html += '<tr>\n'
            html += '<td align="center">'
            html += 'Test Result'
            html += '</td>\n'
            if  self.cd_stress_test_result == "PASS":
                color = "darkgreen"
            elif self.cd_stress_test_result == "N/A":
                color = "darkgrey"
            else:
                color = "red"
            html += '<td align="center" valign="middle" style="background-color:  ' + color + '; color: white;">'
            html += '<b>' + self.cd_stress_test_result + '</b>'
            html += '</td>\n' 
            html += '<td align="center" colspan="2" align="left" width="50%">'
            html += self.cd_stress_test_msg
            html += '</td>\n' 
            html += '</tr>\n'
            html += '</table>\n'
            html += '<br /><br />\n'
        
        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td align="center" colspan="4"><b>Steady-State Stress Calculation</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'F<sub>Md</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.FMd)
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'F<sub>Ml</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.FMl)
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'G<sub>SP</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.GSP)
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'G<sub>RD</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.GRD)
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'C<sub>D</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.CD)
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'C<sub>d</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Cd)
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'C<sub>l</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Cl)
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'P<sub>D</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.PD) + ' Pa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'P<sub>d</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Pd) + ' Pa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'P<sub>l</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Pl) + ' Pa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'S<sub>D</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.SD / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'S<sub>D step</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.SD_step / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'S<sub>d</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Sd / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'S<sub>d step</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Sd_step / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'S<sub>l</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Sl / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'S<sub>l step</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Sl_step / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'S<sub>r</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Sr / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'S<sub>t</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.St / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'S<sub>t step</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.St_step / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'S<sub>a</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Sa / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'S<sub>a step</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Sa_step / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'S<sub>max</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Smax / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'S<sub>max step</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Smax_step / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'VM Criterion'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.VM_criterion / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'VM Criterion step'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.VM_criterion_step / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'VM Test Criterion'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.ss_stress_test_criterion / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Test Result'
        html += '</td>\n' 
        if  self.ss_stress_test_result == "PASS":
            color = "darkgreen"
        elif self.ss_stress_test_result == "N/A":
            color = "darkgrey"
        else:
            color = "red"
        html += '<td align="center" valign="middle" style="background-color:  ' + color + '; color: white;">'
        html += '<b>' + self.ss_stress_test_result + '</b>'
        html += '</td>\n' 
        html += '<td align="center" colspan="2" align="left" width="50%">'
        html += self.ss_stress_test_msg
        html += '</td>\n' 
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br /><br />\n'

        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td align="center" colspan="4"><b>Dynamic Stress Calculation</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'K<sub>t</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Kt)
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'K<sub>t step</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Kt_step)
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'F<sub>T</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.FT)
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'F<sub>E</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.FE)
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'S<sub>o,max</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Somax / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'S<sub>o,max step</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Somax_step / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'S<sub>o,max</sub> Test Criterion'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.dyn_stress_test_criterion / 1000000) + ' MPa'
        html += '</td>\n' 
        html += '<td colspan="2" width="50%">'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Test Result'
        html += '</td>\n' 
        if  self.dyn_stress_test_result == "PASS":
            color = "darkgreen"
        elif self.dyn_stress_test_result == "N/A":
            color = "darkgrey"
        else:
            color = "red"
        html += '<td align="center" valign="middle" style="background-color:  ' + color + '; color: white;">'
        html += '<b>' + self.dyn_stress_test_result + '</b>'
        html += '</td>\n' 
        html += '<td align="center" colspan="2" align="left" width="50%">'
        html += self.dyn_stress_test_msg
        html += '</td>\n' 
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br /><br />\n'

        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td align="center" colspan="4"><b>Hydrostatic Pressure Calculation</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '<td align="center" width="20%"><b>Property</b></td>\n'
        html += '<td align="center" width="30%"><b>Value</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'P<sub>c</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Pc / 1000) + ' kPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'P<sub>t</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Pt / 1000) + ' kPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'P<sub>r</sub>'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.Pr / 1000) + ' kPa'
        html += '</td>\n' 
        html += '<td align="center">'
        html += 'P<sub>r</sub> Test Criterion'
        html += '</td>\n' 
        html += '<td align="center">'
        html += self.SetDecimals(self.press_test_criterion / 1000) + ' kPa'
        html += '</td>\n' 
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center">'
        html += 'Test Result'
        html += '</td>\n' 
        if  self.press_test_result == "PASS":
            color = "darkgreen"
        elif self.press_test_result == "N/A":
            color = "darkgrey"
        else:
            color = "red"
        html += '<td align="center" valign="middle" style="background-color:  ' + color + '; color: white;">'
        html += '<b>' + self.press_test_result + '</b>'
        html += '</td>\n' 
        html += '<td align="center" colspan="2" align="left" width="50%">'
        html += self.press_test_msg
        html += '</td>\n' 
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br /><br />\n'

        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td><b>Notes</b></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td>1. </td>\n'
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br />\n'

        html += '<table width="100%" border="1" cellpadding="5" cellspacing="0">\n'
        html += '<tr>\n'
        html += '<td align="center" width="10%"></td>\n'
        html += '<td align="center" width="10%"></td>\n'
        html += '<td align="center" width="50%"></td>\n'
        html += '<td align="center" width="10%"></td>\n'
        html += '<td align="center" width="10%"></td>\n'
        html += '<td align="center" width="10%"></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td></td>\n'
        html += '<td></td>\n'
        html += '<td></td>\n'
        html += '<td></td>\n'
        html += '<td></td>\n'
        html += '<td></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td></td>\n'
        html += '<td></td>\n'
        html += '<td></td>\n'
        html += '<td></td>\n'
        html += '<td></td>\n'
        html += '<td></td>\n'
        html += '</tr>\n'
        html += '<tr>\n'
        html += '<td align="center"><b>Revision</b></td>\n'
        html += '<td align="center"><b>Date</b></td>\n'
        html += '<td align="center"><b>Description</b></td>\n'
        html += '<td align="center"><b>Done by</b></td>\n'
        html += '<td align="center"><b>Checked by</b></td>\n'
        html += '<td align="center"><b>Approved by</b></td>\n'
        html += '</tr>\n'
        html += '</table>\n'
        html += '<br />\n'
        
        html += '<hr />\n'
        html += '<b style="font-size: 6pt;">Cute Thermowell ver. ' + self.version + '</b>\n'
        html += '<br />\n'
        html += '<b style="font-size: 6pt;">' + self.results_file + '</b>\n'
        html += '</body>\n'
        html += '</html>\n'
        
        file = QFile(self.results_file)
        file.open(QIODevice.WriteOnly)
        file.writeData(str(html))
        file.close()

        return stamp
